@@ -5,7 +5,7 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -88,6 +88,72 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "hastablehdu",
+      UI_KEY_HASTABLEHDU,
+      0,
+      0,
+      "File has at least one table HDU.",
+      UI_GROUP_EXTENSION_INFORMATION,
+      &p->hastablehdu,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "hasimagehdu",
+      UI_KEY_HASIMAGEHDU,
+      0,
+      0,
+      "File has at least one image HDU.",
+      UI_GROUP_EXTENSION_INFORMATION,
+      &p->hasimagehdu,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "listtablehdus",
+      UI_KEY_LISTTABLEHDUS,
+      0,
+      0,
+      "List all table HDUs within the file.",
+      UI_GROUP_EXTENSION_INFORMATION,
+      &p->listtablehdus,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "listimagehdus",
+      UI_KEY_LISTIMAGEHDUS,
+      0,
+      0,
+      "List all image HDUs within the file.",
+      UI_GROUP_EXTENSION_INFORMATION,
+      &p->listimagehdus,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "listallhdus",
+      UI_KEY_LISTALLHDUS,
+      0,
+      0,
+      "List all HDUs within the file.",
+      UI_GROUP_EXTENSION_INFORMATION,
+      &p->listallhdus,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
 
 
 
@@ -100,7 +166,7 @@ struct argp_option program_options[] =
     {
       "remove",
       UI_KEY_REMOVE,
-      "STR",
+      "STR/INT",
       0,
       "Remove extension from input file.",
       UI_GROUP_EXTENSION_MANIPULATION,
@@ -113,7 +179,7 @@ struct argp_option program_options[] =
     {
       "copy",
       UI_KEY_COPY,
-      "STR",
+      "STR/INT",
       0,
       "Copy extension to output file.",
       UI_GROUP_EXTENSION_MANIPULATION,
@@ -126,7 +192,7 @@ struct argp_option program_options[] =
     {
       "cut",
       UI_KEY_CUT,
-      "STR",
+      "STR/INT",
       0,
       "Copy extension to output and remove from input.",
       UI_GROUP_EXTENSION_MANIPULATION,
@@ -164,7 +230,7 @@ struct argp_option program_options[] =
       UI_KEY_ASIS,
       "STR",
       0,
-      "Write the argument string as is into the header.",
+      "Write value as-is (may corrupt FITS file).",
       UI_GROUP_KEYWORD,
       &p->asis,
       GAL_TYPE_STRLL,
@@ -172,6 +238,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "keyvalue",
+      UI_KEY_KEYVALUE,
+      "STR[,STR,...]",
+      0,
+      "Only print the value of requested keyword(s).",
+      UI_GROUP_KEYWORD,
+      &p->keyvalue,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "delete",
       UI_KEY_DELETE,
@@ -188,7 +267,7 @@ struct argp_option program_options[] =
     {
       "rename",
       UI_KEY_RENAME,
-      "STR",
+      "STR,STR",
       0,
       "Rename keyword, keeping value and comments.",
       UI_GROUP_KEYWORD,
@@ -201,7 +280,7 @@ struct argp_option program_options[] =
     {
       "update",
       UI_KEY_UPDATE,
-      "STR",
+      "STR,STR",
       0,
       "Update a keyword value or comments.",
       UI_GROUP_KEYWORD,
@@ -289,12 +368,25 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "printkeynames",
+      UI_KEY_PRINTKEYNAMES,
+      0,
+      0,
+      "Print all keyword names.",
+      UI_GROUP_KEYWORD,
+      &p->printkeynames,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "copykeys",
       UI_KEY_COPYKEYS,
-      "INT:INT",
+      "INT:INT/STR,STR",
       0,
-      "Range of keywords to copy to output HDU.",
+      "Range/Names of keywords to copy to output HDU.",
       UI_GROUP_KEYWORD,
       &p->copykeys,
       GAL_TYPE_STRING,
@@ -328,6 +420,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "wcscoordsys",
+      UI_KEY_WCSCOORDSYS,
+      "STR",
+      0,
+      "Convert WCS coordinate system.",
+      UI_GROUP_KEYWORD,
+      &p->wcscoordsys,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -347,6 +452,21 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "colinfoinstdout",
+      UI_KEY_COLINFOINSTDOUT,
+      0,
+      0,
+      "Column info/metadata when printing to stdout.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->colinfoinstdout,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
 
 
 
