@@ -1143,6 +1143,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+    {
+      "std",
+      UI_KEY_STD,
+      0,
+      0,
+      "Standard dev. of values in object/clump.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
     {
       "median",
       UI_KEY_MEDIAN,
@@ -1368,8 +1382,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "std",
-      UI_KEY_STD,
+      "skystd",
+      UI_KEY_SKYSTD,
       0,
       0,
       "Sky standard deviation (per pixel).",
