@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2021, Free Software Foundation, Inc.
+Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -578,8 +578,9 @@ parse_objects(struct mkcatalog_passparams *pp)
 
                   /* General flux summations. */
                   if(xybin) xybinarr[ pind ]=2;
-                  if(oif[ OCOL_NUM ]) oi[ OCOL_NUM ]++;
-                  if(oif[ OCOL_SUM ]) oi[ OCOL_SUM ] += *V;
+                  if(oif[ OCOL_NUM ])   oi[ OCOL_NUM   ]++;
+                  if(oif[ OCOL_SUM ])   oi[ OCOL_SUM   ] += *V;
+                  if(oif[ OCOL_SUMP2 ]) oi[ OCOL_SUMP2 ] += *V * *V;
 
                   /* Get the necessary clump information. */
                   if(p->clumps && *C>0)
@@ -681,6 +682,7 @@ parse_objects(struct mkcatalog_passparams *pp)
               /* Sky standard deviation based measurements.*/
               if(p->std)
                 {
+                  /* Calculate the variance and save it in the output if necessary. */
                   sval = pp->st_std ? *ST : (p->std->size>1?std[tid]:std[0]);
                   var = p->variance ? sval : sval*sval;
                   if(oif[ OCOL_SUMVAR ] && (!isnan(var)))
@@ -688,6 +690,7 @@ parse_objects(struct mkcatalog_passparams *pp)
                       oi[ OCOL_NUMVAR  ]++;
                       oi[ OCOL_SUMVAR  ] += var;
                     }
+
                   /* For each pixel, we have a sky contribution to the
                      counts and the signal's contribution. The standard
                      deviation in the sky is simply 'sval', but the
@@ -701,7 +704,11 @@ parse_objects(struct mkcatalog_passparams *pp)
                   if(oif[ OCOL_SUM_VAR ] && goodvalue)
                     {
                       varval=p->variance ? var : sval;
-                      if(!isnan(varval)) oi[ OCOL_SUM_VAR ] += varval + fabs(*V);
+                      if(!isnan(varval))
+                        {
+                          oi[ OCOL_SUM_VAR_NUM  ]++;
+                          oi[ OCOL_SUM_VAR      ] += varval + fabs(*V);
+                        }
                     }
                 }
             }
@@ -888,7 +895,7 @@ parse_clumps(struct mkcatalog_passparams *pp)
 
   /* For the extrema columns. */
   if( cif[    CCOL_MINVNUM ] || cif[ CCOL_MINVX ]
-      || cif[ CCOL_MINVX   ] || cif[ CCOL_MINVZ ] )
+      || cif[ CCOL_MINVY   ] || cif[ CCOL_MINVZ ] )
     minima_v=parse_init_extrema(cif, GAL_TYPE_FLOAT64, pp->clumpsinobj, 0);
   if( cif[    CCOL_MAXVNUM ] || cif[ CCOL_MAXVX ]
       || cif[ CCOL_MAXVY   ] || cif[ CCOL_MAXVZ ] )
@@ -980,8 +987,9 @@ parse_clumps(struct mkcatalog_passparams *pp)
                       goodvalue=1;
 
                       /* Fill in the necessary information. */
-                      if(cif[ CCOL_NUM   ]) ci[ CCOL_NUM ]++;
-                      if(cif[ CCOL_SUM   ]) ci[ CCOL_SUM ] += *V;
+                      if(cif[ CCOL_NUM   ]) ci[ CCOL_NUM   ]++;
+                      if(cif[ CCOL_SUM   ]) ci[ CCOL_SUM   ] += *V;
+                      if(cif[ CCOL_SUMP2 ]) ci[ CCOL_SUMP2 ] += *V * *V;
                       if(cif[ CCOL_NUMXY ])
                         ((uint8_t *)(xybin[cind].array))[ pind ] = 2;
 
@@ -1075,7 +1083,10 @@ parse_clumps(struct mkcatalog_passparams *pp)
                         {
                           varval=p->variance ? var : sval;
                           if(!isnan(varval))
-                            ci[ CCOL_SUM_VAR ] += varval + fabs(*V);
+                            {
+                              ci[ CCOL_SUM_VAR_NUM ]++;
+                              ci[ CCOL_SUM_VAR     ] += varval + fabs(*V);
+                            }
                         }
                     }
                 }
