@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -120,8 +120,9 @@ statistics_print_one_row(struct statisticsparams *p)
         sum = sum ? sum : gal_statistics_sum(p->input);              break;
       case UI_KEY_MEDIAN:
         med = med ? med : gal_statistics_median(p->sorted, 0); break;
-      case UI_KEY_MEAN:
       case UI_KEY_STD:
+      case UI_KEY_MEAN:
+      case UI_KEY_QUANTOFMEAN:
         meanstd = meanstd ? meanstd : gal_statistics_mean_std(p->input);
         break;
       case UI_KEY_MODE:
@@ -196,18 +197,34 @@ statistics_print_one_row(struct statisticsparams *p)
 
         /* Not previously calculated. */
         case UI_KEY_QUANTILE:
+          mustfree=1;
           arg = statistics_read_check_args(p);
           out = gal_statistics_quantile(p->sorted, arg, 0);
           break;
 
         case UI_KEY_QUANTFUNC:
+          mustfree=1;
           arg = statistics_read_check_args(p);
           tmpv = gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
                                 NULL, 1, -1, 1, NULL, NULL, NULL);
           *((double *)(tmpv->array)) = arg;
           tmpv = gal_data_copy_to_new_type_free(tmpv, p->input->type);
           out = gal_statistics_quantile_function(p->sorted, tmpv, 0);
+          gal_data_free(tmpv);
           break;
+
+        case UI_KEY_QUANTOFMEAN:
+          mustfree=1;
+          tmpv=statistics_pull_out_element(meanstd, 0);
+          out = gal_statistics_quantile_function(p->sorted, tmpv, 0);
+          gal_data_free(tmpv);
+          break;
+
+        /* The option isn't recognized. */
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we "
+                "can address the problem. Operation code %d not recognized",
+                __func__, PACKAGE_BUGREPORT, tmp->v);
         }
 
       /* Print the number. Note that we don't want any extra white space
@@ -285,7 +302,8 @@ statistics_interpolate_and_write(struct statisticsparams *p,
   /* Write the values. */
   gal_tile_full_values_write(values, &cp->tl, !p->ignoreblankintiles,
                              output, NULL, PROGRAM_NAME);
-  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
+                              p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
                             "STATISTICS-CONFIG", output, "0");
 }
@@ -664,7 +682,8 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
   /* Write the configuration information if we have a FITS output. */
   if(isfits)
     {
-      gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+      gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
+                                  p->cp.quiet);
       gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
                                 "STATISTICS-CONFIG", output, "0");
     }
@@ -838,7 +857,8 @@ histogram_2d(struct statisticsparams *p)
       /* Write the output. */
       output=statistics_output_name(p, suf, &isfits);
       gal_fits_img_write(img, output, NULL, PROGRAM_STRING);
-      gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+      gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
+                                  p->cp.quiet);
       gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
                                 "STATISTICS-CONFIG", output, "0");
 
