@@ -656,7 +656,18 @@ table_select_by_position(struct tableparams *p)
   char **strarr;
   gal_data_t *col;
   size_t i, start, end;
-  double *darr = p->rowlimit ? p->rowlimit->array : NULL;
+  double *darr = p->rowrange ? p->rowrange->array : NULL;
+
+  /* If the head or tail values are given and are larger than the number of
+     rows, just set them to the number of rows (print the all the final
+     rows). This is how the 'head' and 'tail' programs of GNU Coreutils
+     operate. */
+  p->head = ( ((p->head!=GAL_BLANK_SIZE_T) && (p->head > p->table->size))
+              ? p->table->size
+              : p->head );
+  p->tail = ( ((p->tail!=GAL_BLANK_SIZE_T) && (p->tail > p->table->size))
+              ? p->table->size
+              : p->tail );
 
   /* Random row selection (by position, not value). This step is
      independent of the other operations of this function, so as soon as
@@ -674,15 +685,16 @@ table_select_by_position(struct tableparams *p)
       return;
     }
 
-  /* Sanity check  */
-  if(p->rowlimit)
+  /* Make sure the given values to '--rowrange' are within the number of
+     rows until this point. */
+  if(p->rowrange)
     {
       if(darr[0]>=p->table->size)
-        error(EXIT_FAILURE, 0, "the first value to '--rowlimit' (%g) "
+        error(EXIT_FAILURE, 0, "the first value to '--rowrange' (%g) "
               "is larger than the number of rows (%zu)",
               darr[0]+1, p->table->size);
       else if( darr[1]>=p->table->size )
-        error(EXIT_FAILURE, 0, "the second value to '--rowlimit' (%g) "
+        error(EXIT_FAILURE, 0, "the second value to '--rowrange' (%g) "
               "is larger than the number of rows (%zu)",
               darr[1]+1, p->table->size);
     }
@@ -699,9 +711,9 @@ table_select_by_position(struct tableparams *p)
         {
           /* Parse the rows and free extra pointers. */
           strarr=col->array;
-          if(p->rowlimit)
+          if(p->rowrange)
             {
-              /* Note that the given values to '--rowlimit' started from 1,
+              /* Note that the given values to '--rowrange' started from 1,
                  but in 'ui.c' we subtracted one from it (so at this stage,
                  it starts from 0). */
               start = darr[0];
@@ -713,7 +725,7 @@ table_select_by_position(struct tableparams *p)
             {
               /* Set the starting and ending indexs to free the allocated
                  space of each string. */
-              start = p->head!=GAL_BLANK_SIZE_T ? p->head        : 0;
+              start = p->head!=GAL_BLANK_SIZE_T ? p->head : 0;
               end   = ( p->head!=GAL_BLANK_SIZE_T
                         ? p->table->size
                         : p->table->size - p->tail );
@@ -722,7 +734,7 @@ table_select_by_position(struct tableparams *p)
         }
 
       /* Make the final adjustment. */
-      if(p->rowlimit)
+      if(p->rowrange)
         {
           /* Move the values up to the top and correct the size. */
           col->size=darr[1]-darr[0]+1;
@@ -760,8 +772,8 @@ table_catcolumn(struct tableparams *p)
 {
   size_t counter=1;
   gal_list_str_t *filell, *hdull;
-  gal_data_t *tocat, *final, *newcol;
   char *tmpname, *hdu=NULL, cstr[100];
+  gal_data_t *col, *tocat, *final, *newcol;
   struct gal_options_common_params *cp=&p->cp;
 
   /* Go over all the given files. */
@@ -804,13 +816,20 @@ table_catcolumn(struct tableparams *p)
         for(newcol=tocat; newcol!=NULL; newcol=newcol->next)
           if(newcol->name)
             {
-              /* Add the counter suffix to the column name. */
-              sprintf(cstr, "-%zu", counter);
-              tmpname=gal_checkset_malloc_cat(newcol->name, cstr);
+              /* If the new column's name is identical (case-insensitive)
+                 to an existing name in the existing table so far, we need
+                 to add a suffix. */
+              for(col=p->table; col!=NULL; col=col->next)
+                if( col->name && strcasecmp(col->name, newcol->name)==0 )
+                  {
+                    /* Add the counter suffix to the column name. */
+                    sprintf(cstr, "-%zu", counter);
+                    tmpname=gal_checkset_malloc_cat(newcol->name, cstr);
 
-              /* Free the old name and put in the new one. */
-              free(newcol->name);
-              newcol->name=tmpname;
+                    /* Free the old name and put in the new one. */
+                    free(newcol->name);
+                    newcol->name=tmpname;
+                  }
             }
 
       /* Find the final column of the main table and add this table.*/
@@ -858,7 +877,7 @@ table_catrows_prepare(struct tableparams *p)
   char **strarr;
   char *hdu=NULL;
   int tableformat;
-  gal_data_t *tmp, *out=NULL;
+  gal_data_t *ocol, *tmp;
   size_t i, nrows=p->table->size;
   gal_list_str_t *filell, *hdull;
   size_t numcols, numrows, filledrows=p->table->size;
@@ -873,19 +892,23 @@ table_catrows_prepare(struct tableparams *p)
       nrows+=numrows;
     }
 
-  /* Allocate the new table with the necessary number of rows, then reverse
-     the newly allocated table (its columns were added in a
-     first-in-first-out list).*/
+  /* Change the 'array' component of each column (to preserve the column's
+     'gal_data_t' pointer; since previous preparations have kept that
+     pointer for next steps in some cases like column arithmetic). To do
+     this, we will first allocate a full 'gal_data_t', copy the contents,
+     free the contents of the original column, then replace them with the
+     larger array, and free the temporary 'gal_data_t'. */
   for(tmp=p->table; tmp!=NULL; tmp=tmp->next)
     {
-      /* Allocate the new column. */
-      gal_list_data_add_alloc(&out, NULL, tmp->type, 1, &nrows, NULL,
-                              0, p->cp.minmapsize, p->cp.quietmmap,
-                              tmp->name, tmp->unit, tmp->comment);
+      /* Allocate a temporary dataset (we just want its allocated space,
+         not the actual 'gal_data_t' pointer)! */
+      ocol=gal_data_alloc(NULL, tmp->type, 1, &nrows, NULL,
+                          0, p->cp.minmapsize, p->cp.quietmmap,
+                          tmp->name, tmp->unit, tmp->comment);
 
       /* Put the full contents of the existing column into the new
          column: this will be the first set of rows,  */
-      memcpy(out->array, tmp->array, tmp->size*gal_type_sizeof(tmp->type));
+      memcpy(ocol->array, tmp->array, tmp->size*gal_type_sizeof(tmp->type));
 
       /* If the column type is a string, we should set the input pointers
          to NULL to avoid freeing them later. */
@@ -894,12 +917,22 @@ table_catrows_prepare(struct tableparams *p)
           strarr=tmp->array;
           for(i=0;i<tmp->size;++i) strarr[i]=NULL;
         }
+
+      /* Free the contents of the current column (while keeping its
+         pointer), and replace those of the 'ocol' dataset. */
+      gal_data_free_contents(tmp);
+      tmp->comment=ocol->comment; ocol->comment=NULL;
+      tmp->array=ocol->array;     ocol->array=NULL;
+      tmp->dsize=ocol->dsize;     ocol->dsize=NULL;
+      tmp->name=ocol->name;       ocol->name=NULL;
+      tmp->unit=ocol->unit;       ocol->unit=NULL;
+      tmp->size=ocol->size;
+
+      /* Free the 'ocol' dataset. */
+      gal_data_free(ocol);
     }
-  gal_list_data_reverse(&out);
 
   /* Clean up and return. */
-  gal_list_data_free(p->table);
-  p->table=out;
   return filledrows;
 }
 
@@ -1189,7 +1222,7 @@ table(struct tableparams *p)
   if(p->sort) table_sort(p);
 
   /* If the output number of rows is limited, apply them. */
-  if( p->rowlimit
+  if( p->rowrange
       || p->rowrandom
       || p->head!=GAL_BLANK_SIZE_T
       || p->tail!=GAL_BLANK_SIZE_T )
