@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -227,6 +227,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
 static void
 ui_read_check_only_options(struct tableparams *p)
 {
+  size_t i;
   double *darr;
   gal_data_t *tmp;
 
@@ -274,15 +275,55 @@ ui_read_check_only_options(struct tableparams *p)
     }
 
   /* Make sure only one of the positional row selection operations is
-     called in this run.*/
-  if(p->rowlimit
-     && p->rowrandom
-     && p->head!=GAL_BLANK_SIZE_T
-     && p->tail!=GAL_BLANK_SIZE_T)
+     called in one run. */
+  if( (p->rowrange!=NULL)
+      + (p->rowrandom!=0)
+      + (p->head!=GAL_BLANK_SIZE_T)
+      + (p->tail!=GAL_BLANK_SIZE_T) > 1 )
     error(EXIT_FAILURE, 0, "only one of the following options can be "
-          "called in one run: '--head', '--tail', '--rowlimit' and "
+          "called in one run: '--head', '--tail', '--rowrange' and "
           "'--rowrandom'");
 
+  /* Make sure the value to '--rowrange' is in the correct format. */
+  if(p->rowrange)
+    {
+      /* There should only be two values. */
+      if(p->rowrange->size!=2)
+        error(EXIT_FAILURE, 0, "only two should be given to "
+              "'--rowrange' (the top and bottom row numbers specifying "
+              "your desired range)");
+
+      /* Do individual checks. */
+      darr=p->rowrange->array;
+      for(i=0;i<p->rowrange->size;++i)
+        {
+          /* Make sure it isn't 0 or negative. */
+          if( darr[i]<=0 )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowrange') "
+                  "is smaller than, or equal to, zero! This option's "
+                  "values are row-counters (starting from 1), so they "
+                  "must be positive integers", darr[i]);
+
+          /* Make sure its an integer. */
+          if( darr[i] != (size_t)(darr[i]) )
+            error(EXIT_FAILURE, 0, "%g (value given to '--rowrange') is "
+                  "not an integer! This option's values are row-counters "
+                  "so they must be integers.", darr[i]);
+
+          /* Subtract 1 from the value, so it counts from 0. */
+          --darr[i];
+        }
+
+      /* Make sure that the first value is smaller than the second. */
+      if( darr[0] > darr[1] )
+        error(EXIT_FAILURE, 0, "the first value to '--rowrange' (%g) is "
+              "larger than the second (%g). This option's values defines "
+              "a row-counter interval, assuming the first value is the top "
+              "of the desired interval (smaller row counter) and the second "
+              "value is the bottom of the desired interval (larger row "
+              "counter)", darr[0], darr[1]);
+    }
+
   /* If '--colmetadata' is given, make sure none of the given options have
      more than three values. */
   if(p->colmetadata)
@@ -540,6 +581,7 @@ ui_print_info_exit(struct tableparams *p)
   if(p->filename==NULL) p->filename="Standard-input";
   gal_list_str_free(lines, 1);
 
+
   /* If there was no actual data in the file, then inform the user */
   if(allcols==NULL)
     error(EXIT_FAILURE, 0, "%s: no usable data rows", p->filename);
@@ -576,13 +618,15 @@ ui_print_info_exit(struct tableparams *p)
    has comma-separated values. If they do then the list will be updated to
    be fully separate. */
 static void
-ui_columns_prepare(struct tableparams *p)
+ui_columns_prepare(struct tableparams *p, gal_list_str_t *lines)
 {
+  int tableformat;
   gal_data_t *strs;
   char *c, **strarr;
-  size_t i, totcalled=0;
+  gal_data_t *colinfo=NULL;
   struct column_pack *node, *last;
-  gal_list_str_t *tmp, *toread=NULL;
+  gal_list_str_t *tmp, *colstoread=NULL;
+  size_t i, totcalled=0, numcols, numrows;
 
   /* Go over the whole original list (where each node may have more than
      one value separated by a comma. */
@@ -604,37 +648,49 @@ ui_columns_prepare(struct tableparams *p)
       /* Go over all the given colum names/numbers. */
       for(i=0;i<strs->size;++i)
         {
-          /* See if this is an arithmetic column to be processed, or the
-             contents should just be printed. */
+          /* See if this is an arithmetic column to be processed, or its
+             just a "simple" column (where  */
           if(!strncmp(strarr[i], ARITHMETIC_CALL, ARITHMETIC_CALL_LENGTH))
             {
+              /* Arithmetic operations may be done on columns from other
+                 files (for example with '--catcolumnfile'). We therefore
+                 need to check if the requested column is in the main input
+                 file or not. If not, it should be set when column
+                 arithmetic starts. To do this, we need to get the input's
+                 column information. */
+              if(colinfo==NULL)
+                colinfo=gal_table_info(p->filename, p->cp.hdu, lines,
+                                       &numcols, &numrows, &tableformat);
+
               /* If this is the first arithmetic operation and the user has
                  already asked for some columns, we'll need to put all
                  previously requested simply-printed columns into an
                  'outcols' structure, then add this arithmetic operation's
                  'outcols'. */
-              if(p->outcols==NULL && toread)
+              if(p->outcols==NULL && colstoread)
                 {
                   /* Allocate an empty structure and set the necessary
                      pointers. */
                   node=ui_outcols_add_new_to_end(&p->outcols);
                   node->start=0;
-                  node->numsimple=gal_list_str_number(toread);
+                  node->numsimple=gal_list_str_number(colstoread);
                   totcalled=node->numsimple;
                 }
 
               /* Add a new column pack, then read all the tokens (while
                  specifying which columns it needs). */
               node=ui_outcols_add_new_to_end(&p->outcols);
-              arithmetic_init(p, &node->arith, &toread, &totcalled,
-                              strarr[i]+ARITHMETIC_CALL_LENGTH);
+              arithmetic_init(p, &node->arith, &colstoread, &totcalled,
+                              strarr[i]+ARITHMETIC_CALL_LENGTH, colinfo,
+                              numcols);
               free(strarr[i]);
             }
+
           /* This is a simple column (no change in values). */
           else
             {
               /* Add this column to the list of columns to read. */
-              gal_list_str_add(&toread, strarr[i], 0);
+              gal_list_str_add(&colstoread, strarr[i], 0);
 
               /* See if we have packaged the output columns. */
               if(p->outcols)
@@ -700,11 +756,15 @@ ui_columns_prepare(struct tableparams *p)
     }
   */
 
-  /* Delete the old list, then reverse the 'toread' list, and put it into
-     'p->columns'. */
+  /* Free the information from all the columns (that may have been gathered
+     if it was necessary). */
+  if(colinfo) gal_data_array_free(colinfo, numcols, 0);
+
+  /* Delete the old list, then reverse the 'colstoread' list, and put it
+     into 'p->columns'. */
   gal_list_str_free(p->columns, 1);
-  gal_list_str_reverse(&toread);
-  p->columns=toread;
+  gal_list_str_reverse(&colstoread);
+  p->columns=colstoread;
 }
 
 
@@ -730,6 +790,64 @@ ui_check_select_sort_read_col_ind(char *string)
 
 
 
+/* Fill replace '--noblank' with all columns (if it is given '_all'). */
+static void
+ui_noblank_prepare(struct tableparams *p, size_t numcols)
+{
+  size_t i, zero=0;
+  gal_list_str_t *tmp;
+  char name[50]; /* No table will have a 50 digit number of columns! */
+
+  /* Merge all the possible calls of 'noblank' into one. */
+  gal_options_merge_list_of_csv(&p->noblankll);
+
+  /* Initialize the 'noblank' list of (empty!) datasets (we only need the
+     column names, this is done to conform with the general selection
+     mechanism, where some selection methods need values). */
+  p->noblank=NULL;
+
+  /* In case it is given an '_all' option (for all columns to be
+     checked). */
+  if(gal_list_str_number(p->noblankll)==1
+     && !strcmp(p->noblankll->v,"_all"))
+    {
+      /* Add all the columns to the final list of datasets. */
+      for(i=0;i<numcols;++i)
+        {
+          sprintf(name, "%zu", i+1);
+          gal_list_data_add_alloc(&p->noblank, NULL, GAL_TYPE_FLOAT64, 1,
+                                  &zero, NULL, 0, -1, 1, name, NULL, NULL);
+        }
+    }
+
+  /* Only certain columns are requested (we should just put the strings in
+     the desired format of a list of datasets with names, not a single
+     dataset with type string and many elements). */
+  else
+    {
+      for(tmp=p->noblankll; tmp!=NULL; tmp=tmp->next)
+        gal_list_data_add_alloc(&p->noblank, NULL, GAL_TYPE_FLOAT64, 1,
+                                &zero, NULL, 0, -1, 1, tmp->v, NULL, NULL);
+    }
+
+  /* Reverse the final output list of columns. */
+  gal_list_data_reverse(&p->noblank);
+
+  /* Clean up the list that we don't need any more. */
+  gal_list_str_free(p->noblankll, 1);
+  p->noblankll=NULL;
+
+  /* For a check:
+  gal_data_t *dtmp;
+  for(dtmp=p->noblank;dtmp!=NULL;dtmp=dtmp->next)
+    printf("col: %s\n", dtmp->name);
+  exit(0);
+  */
+}
+
+
+
+
 
 /* See if row selection or sorting needs any extra columns to be read. */
 static void
@@ -749,10 +867,11 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
   gal_data_t *dtmp, *allcols, *inpolytmp=NULL, *outpolytmp=NULL;
 
   /* Important note: these have to be in the same order as the 'enum
-     select_types' in 'main.h'. We'll fill the two polygon elements
-     later. */
+     select_types' in 'main.h'. We'll fill the NULL components
+     afterwards. */
   gal_data_t *select[SELECT_TYPE_NUMBER]={p->range, NULL, NULL,
-                                          p->equal, p->notequal};
+                                          p->equal, p->notequal,
+                                          NULL};
 
 
   /* The inpolygon dataset is currently a single dataset with two elements
@@ -778,14 +897,32 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
     }
 
 
+  /* Get all the input table's column information. */
+  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
+                         &numrows, &tableformat);
+
+
+  /* If '--noblank' is given an '_all', we should replace it with the
+     counters of all the input table columns. */
+  if(p->noblankll)
+    {
+      ui_noblank_prepare(p, numcols);
+      select[SELECT_TYPE_NOBLANK]=p->noblank;
+    }
+
+
   /* Allocate necessary spaces. */
   if(p->selection)
     {
+      /* Find the number of columns that are needed for selection. */
       *nselect = ( gal_list_data_number(p->range)
                    + gal_list_data_number(inpolytmp)
                    + gal_list_data_number(outpolytmp)
                    + gal_list_data_number(p->equal)
-                   + gal_list_data_number(p->notequal) );
+                   + gal_list_data_number(p->notequal)
+                   + gal_list_data_number(p->noblank) );
+
+      /* Allocate the necessary arrays. */
       selectind=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
                                      __func__, "selectind");
       selecttype=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
@@ -794,7 +931,12 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
                                         __func__, "selectindout");
       selecttypeout=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
                                          __func__, "selecttypeout");
+
+      /* Initialize them to blank. */
       sf=(s=selectindout)+*nselect; do *s++=GAL_BLANK_SIZE_T; while(s<sf);
+
+      /* Set the output pointers (they have already been initialized to
+         NULL by the caller function). */
       *selectindout_out=selectindout;
       *selecttypeout_out=selecttypeout;
     }
@@ -813,11 +955,6 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
         }
 
 
-  /* Get all the column information. */
-  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
-                         &numrows, &tableformat);
-
-
   /* If the values are numbers, we'll check if the given value is less than
      the total number of columns. Just note that the indexs count from
      zero. */
@@ -1049,8 +1186,6 @@ ui_check_select_sort_after(struct tableparams *p, size_t nselect,
 static void
 ui_preparations(struct tableparams *p)
 {
-  double *darr;
-  size_t i, *colmatch;
   gal_list_str_t *lines;
   size_t nselect=0, origoutncols=0;
   size_t sortindout=GAL_BLANK_SIZE_T;
@@ -1063,17 +1198,17 @@ ui_preparations(struct tableparams *p)
     ui_print_info_exit(p);
 
 
-  /* Prepare the column names. */
-  ui_columns_prepare(p);
-
-
   /* If the input is from stdin, save it as 'lines'. */
   lines=gal_options_check_stdin(p->filename, p->cp.stdintimeout, "input");
 
 
+  /* Prepare the column names. */
+  ui_columns_prepare(p, lines);
+
+
   /* If any kind of row-selection is requested set 'p->selection' to 1. */
   p->selection = ( p->range || p->inpolygon || p->outpolygon || p->equal
-                   || p->notequal );
+                   || p->notequal || p->noblankll );
 
 
   /* If row sorting or selection are requested, see if we should read any
@@ -1085,17 +1220,17 @@ ui_preparations(struct tableparams *p)
 
   /* If we have any arithmetic operations, we need to make sure how many
      columns match every given column name. */
-  colmatch = ( p->outcols
-               ? gal_pointer_allocate(GAL_TYPE_SIZE_T,
-                                      gal_list_str_number(p->columns), 1,
-                                      __func__, "colmatch")
-               : NULL);
+  p->colmatch = ( p->outcols
+                  ? gal_pointer_allocate(GAL_TYPE_SIZE_T,
+                                         gal_list_str_number(p->columns),
+                                         1, __func__, "p->colmatch")
+                  : NULL);
 
 
   /* Read the necessary columns. */
   p->table=gal_table_read(p->filename, cp->hdu, lines, p->columns,
-                          cp->searchin, cp->ignorecase, cp->minmapsize,
-                          p->cp.quietmmap, colmatch);
+                          cp->searchin, cp->ignorecase, cp->numthreads,
+                          cp->minmapsize, p->cp.quietmmap, p->colmatch);
   if(p->filename==NULL) p->filename="stdin";
   gal_list_str_free(lines, 1);
 
@@ -1114,80 +1249,17 @@ ui_preparations(struct tableparams *p)
           "non-blank lines)", p->filename);
 
 
-  /* Set the final indexs. */
-  if(p->outcols)
-    arithmetic_indexs_final(p, colmatch);
-
-
-  /* Now that the data columns are ready, we can free the string linked
-     list. */
-  gal_list_str_free(p->columns, 1);
-  p->columns=NULL;
-
-
   /* Make sure the (possible) output name is writable. */
   gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
 
 
-  /* If the head or tail values are given and are larger than the number of
-     rows, just set them to the number of rows (print the all the final
-     rows). This is how the 'head' and 'tail' programs of GNU Coreutils
-     operate. */
-  p->head = ( ((p->head!=GAL_BLANK_SIZE_T) && (p->head > p->table->size))
-              ? p->table->size
-              : p->head );
-  p->tail = ( ((p->tail!=GAL_BLANK_SIZE_T) && (p->tail > p->table->size))
-              ? p->table->size
-              : p->tail );
-
-  /* If rows are given, do some sanity checks and make sure that they are
-     within the table's limits. */
-  if(p->rowlimit)
-    {
-      /* There should only be two values. */
-      if(p->rowlimit->size!=2)
-        error(EXIT_FAILURE, 0, "only two should be given to "
-              "'--rowlimit' (the top and bottom row numbers specifying "
-              "your desired range)");
-
-      /* Do individual checks. */
-      darr=p->rowlimit->array;
-      for(i=0;i<p->rowlimit->size;++i)
-        {
-          /* Make sure it isn't 0 or negative. */
-          if( darr[i]<=0 )
-            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') "
-                  "is smaller than, or equal to, zero! This option's "
-                  "values are row-counters (starting from 1), so they "
-                  "must be positive integers", darr[i]);
-
-          /* Make sure its an integer. */
-          if( darr[i] != (size_t)(darr[i]) )
-            error(EXIT_FAILURE, 0, "%g (value given to '--rowlimit') is "
-                  "not an integer! This option's values are row-counters "
-                  "so they must be integers.", darr[i]);
-
-          /* Subtract 1 from the value, so it counts from 0. */
-          --darr[i];
-        }
-
-      /* Make sure that the first value is smaller than the second. */
-      if( darr[0] > darr[1] )
-        error(EXIT_FAILURE, 0, "the first value to '--rowlimit' (%g) is "
-              "larger than the second (%g). This option's values defines "
-              "a row-counter interval, assuming the first value is the top "
-              "of the desired interval (smaller row counter) and the second "
-              "value is the bottom of the desired interval (larger row "
-              "counter)", darr[0], darr[1]);
-    }
-
   /* If random rows are desired, we need to define a GSL random number
      generator structure. */
   if(p->rowrandom)
     p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 
+
   /* Clean up. */
-  free(colmatch);
   if(selectindout) free(selectindout);
   if(selecttypeout) free(selecttypeout);
 }
@@ -1265,6 +1337,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p)
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
 
+
   /* Let the user know basic information if necessary (for example when a
      random number generator has been used). */
   if(p->rng && !p->cp.quiet)
@@ -1309,8 +1382,14 @@ ui_free_report(struct tableparams *p)
   free(p->cp.output);
   ui_outcols_free(p->outcols);
   gal_list_data_free(p->table);
-  gal_list_data_free(p->colmetadata);
+  if(p->wcshdu) free(p->wcshdu);
+  gal_list_data_free(p->noblank);
+  gal_list_str_free(p->columns, 1);
+  if(p->colmatch) free(p->colmatch);
   if(p->colarray) free(p->colarray);
+  gal_list_data_free(p->colmetadata);
+  gal_list_str_free(p->catcolumnhdu, 1);
+  gal_list_str_free(p->catcolumnfile, 1);
 
   /* If a random number generator was allocated, free it. */
   if(p->rng) gsl_rng_free(p->rng);
