@@ -71,6 +71,15 @@ __BEGIN_C_DECLS  /* From C++ preparations */
                                      | GAL_ARITHMETIC_FLAG_FREE    \
                                      | GAL_ARITHMETIC_FLAG_NUMOK )
 
+/* Operator fixed strings. */
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX     "load-col-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_HDU        "-hdu-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_FILE       "-from-"
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX_LEN strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_PREFIX)
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_HDU_LEN    strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_HDU)
+#define GAL_ARITHMETIC_OPSTR_LOADCOL_FILE_LEN   strlen(GAL_ARITHMETIC_OPSTR_LOADCOL_FILE)
+
+
 
 /* Identifiers for each operator. */
 enum gal_arithmetic_operators
@@ -129,6 +138,9 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_COUNTS_TO_MAG,/* Counts to magnitude. */
   GAL_ARITHMETIC_OP_MAG_TO_COUNTS,/* Magnitude to counts. */
   GAL_ARITHMETIC_OP_COUNTS_TO_JY, /* Counts to Janskys with AB-mag zeropoint. */
+  GAL_ARITHMETIC_OP_JY_TO_COUNTS, /* Janskys to Counts with AB-mag zeropoint. */
+  GAL_ARITHMETIC_OP_MAG_TO_JY,    /* Magnitude to Janskys. */
+  GAL_ARITHMETIC_OP_JY_TO_MAG,    /* Janskys to Magnitude. */
   GAL_ARITHMETIC_OP_AU_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
   GAL_ARITHMETIC_OP_PC_TO_AU,     /* Parsecs (PC) to Astronomical units (AU). */
   GAL_ARITHMETIC_OP_LY_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
@@ -143,6 +155,8 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_MEANVAL,      /* Mean value of array.                  */
   GAL_ARITHMETIC_OP_STDVAL,       /* Standard deviation value of array.    */
   GAL_ARITHMETIC_OP_MEDIANVAL,    /* Median value of array.                */
+  GAL_ARITHMETIC_OP_UNIQUE,       /* Only return unique elements.          */
+  GAL_ARITHMETIC_OP_NOBLANK,      /* Only keep non-blank elements.         */
 
   GAL_ARITHMETIC_OP_MIN,          /* Minimum per pixel of multiple arrays. */
   GAL_ARITHMETIC_OP_MAX,          /* Maximum per pixel of multiple arrays. */
@@ -160,9 +174,13 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_MKNOISE_SIGMA,/* Fixed-sigma noise to every element.   */
   GAL_ARITHMETIC_OP_MKNOISE_POISSON,/* Poission noise on every element.    */
   GAL_ARITHMETIC_OP_MKNOISE_UNIFORM,/* Uniform noise on every element.     */
+  GAL_ARITHMETIC_OP_RANDOM_FROM_HIST,/* Randoms from a histogram (uniform).*/
+  GAL_ARITHMETIC_OP_RANDOM_FROM_HIST_RAW,/* Randoms from a histogram (raw).*/
 
   GAL_ARITHMETIC_OP_SIZE,         /* Size of the dataset along an axis     */
 
+  GAL_ARITHMETIC_OP_STITCH,       /* Stitch multiple datasets together.    */
+
   GAL_ARITHMETIC_OP_TO_UINT8,     /* Convert to uint8_t.                   */
   GAL_ARITHMETIC_OP_TO_INT8,      /* Convert to int8_t.                    */
   GAL_ARITHMETIC_OP_TO_UINT16,    /* Convert to uint16_t.                  */
@@ -187,6 +205,10 @@ gal_arithmetic_operator_string(int operator);
 int
 gal_arithmetic_set_operator(char *string, size_t *num_operands);
 
+gal_data_t *
+gal_arithmetic_load_col(char *str, int searchin, int ignorecase,
+                        size_t minmapsize, int quietmmap);
+
 gal_data_t *
 gal_arithmetic(int operator, size_t numthreads, int flags, ...);
 
