@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -58,13 +58,38 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
    Note that other than the character strings (column name, units and
    comments), nothing in the data structure(s) will be allocated by this
-   function for the actual data (e.g., the 'array' or 'dsize' elements). */
+   function for the actual data (e.g., the 'array' or 'dsize' elements).
+
+   Here are the gal_data_t structure elements that are used in 'allcols':
+
+            *restrict array -> Blank value (if present).
+                       type -> Type of column data.
+                       ndim -> Blank number of dimensions (1)
+                     *dsize -> Blank dimension lengths (1)
+                       size -> Blank total size (1)
+                  quietmmap -> ------------
+                  *mmapname -> ------------
+                 minmapsize -> Repeat (FITS Binary 'TFORM')
+                       nwcs -> ------------
+                       *wcs -> ------------
+                       flag -> 'GAL_TABLEINTERN_FLAG_*' macros.
+                     status -> ------------
+                      *name -> Column name.
+                      *unit -> Column unit.
+                   *comment -> Column comments.
+                   disp_fmt -> 'GAL_TABLE_DISPLAY_FMT' macros.
+                 disp_width -> To keep width of string columns.
+             disp_precision -> ------------
+                      *next -> ------------
+                     *block -> ------------
+
+*/
 gal_data_t *
 gal_table_info(char *filename, char *hdu, gal_list_str_t *lines,
                size_t *numcols, size_t *numrows, int *tableformat)
 {
   /* Get the table format and size (number of columns and rows). */
-  if(filename && gal_fits_name_is_fits(filename))
+  if(filename && gal_fits_file_recognized(filename))
     return gal_fits_tab_info(filename, hdu, numcols, numrows, tableformat);
   else
     {
@@ -90,6 +115,9 @@ gal_table_print_info(gal_data_t *allcols, size_t numcols, size_t numrows)
   int Nw=3, nw=4, uw=5, tw=4;   /* Initial width from label's width */
   char *name, *unit, *comment;
 
+  /* If there aren't any columns, there is no need to print anything. */
+  if(numcols==0) return;
+
   /* Set the widths to print the column information. The width for the
      column number can easily be identified from the logarithm of the
      number of columns. */
@@ -126,12 +154,14 @@ gal_table_print_info(gal_data_t *allcols, size_t numcols, size_t numrows)
       printf("%-*zu%-*s%-*s%-*s%s\n", Nw, i+1,
              nw, name ? name : GAL_BLANK_STRING ,
              uw, unit ? unit : GAL_BLANK_STRING ,
-             tw, gal_type_name(allcols[i].type, 1),
+             tw,
+             allcols[i].type ? gal_type_name(allcols[i].type, 1) : "--",
              comment ? comment : GAL_BLANK_STRING);
     }
 
   /* Print the number of rows. */
-  printf("--------\nNumber of rows: %zu\n--------\n", numrows);
+  if(numrows!=GAL_BLANK_SIZE_T)
+    printf("--------\nNumber of rows: %zu\n--------\n", numrows);
 }
 
 
@@ -410,7 +440,8 @@ gal_table_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
 gal_data_t *
 gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
                gal_list_str_t *cols, int searchin, int ignorecase,
-               size_t minmapsize, int quietmmap, size_t *colmatch)
+               size_t numthreads, size_t minmapsize, int quietmmap,
+               size_t *colmatch)
 {
   int tableformat;
   gal_list_sizet_t *indexll;
@@ -429,15 +460,8 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
                                    ignorecase, filename, hdu, colmatch);
 
   /* Depending on the table format, read the columns into the output
-     structure. Note that the functions here pop each index, read/store the
-     desired column and pop the next, so after these functions, the output
-     linked list will have the opposite order of its input 'indexll'
-     list. So before calling any of them, we will first reverse the
-     'indexll' list, so the output data structure list will have the same
-     order as the input list of desired columns. Also note that after these
-     functions, the 'indexll' will be all freed (each popped element is
-     actually freed).*/
-  gal_list_sizet_reverse(&indexll);
+     structure. Also note that after these functions, the 'indexll' will be
+     all freed (each popped element is actually freed).*/
   switch(tableformat)
     {
     case GAL_TABLE_FORMAT_TXT:
@@ -448,7 +472,7 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
     case GAL_TABLE_FORMAT_AFITS:
     case GAL_TABLE_FORMAT_BFITS:
       out=gal_fits_tab_read(filename, hdu, numrows, allcols, indexll,
-                            minmapsize, quietmmap);
+                            numthreads, minmapsize, quietmmap);
       break;
 
     default:
